                   // Fichier genere avec ./genere -t2 Base
#include "Base.h"

//--
template <class T1, class T2>
Base<T1,T2>::Base(void) : _baseObject(T1()), _basePointer((T2*)NULL)
{
}

//--
template <class T1, class T2>
Base<T1,T2>::Base(const Base<T1,T2>& aB)
{
 _copy(aB);
}

//--
template <class T1, class T2>
Base<T1,T2>& Base<T1,T2>::operator=(const Base<T1,T2>& aB)
{
 if (this != &aB)
 {
  _destroy();
  _copy(aB);
 }
 return *this;
}

//--
template <class T1, class T2>
Base<T1,T2>::~Base(void)
{
 _destroy();
}

//--
template <class T1, class T2>
bool operator==(const Base<T1,T2>& aB1,
                const Base<T1,T2>& aB2)
{
 return aB1.isEqualTo(aB2);
}

//--
template <class T1, class T2>
bool operator!=(const Base<T1,T2>& aB1,
                const Base<T1,T2>& aB2)
{
 return !(aB1==aB2);
}

//--
template <class T1, class T2>
void Base<T1,T2>::setBaseObject(const T1& object)
{
 _baseObject = object;
}

template <class T1, class T2>
const T1& Base<T1,T2>::getBaseObject(void) const
{
 return _baseObject;
}

//--
template <class T1, class T2>
void Base<T1,T2>::setBasePointer(const T2* pointer)
{
 _destroy();
 if (pointer) _basePointer = new T2(*pointer);
 else         _basePointer = NULL;
}

template <class T1, class T2>
const T2* Base<T1,T2>::getBasePointer(void) const
{
 return _basePointer;
}

//--
template <class T1, class T2>
ostream& operator<<(ostream& os, const Base<T1,T2>& aB)
{
 aB.display(os);
 return os;
}

//--
template <class T1, class T2>
void Base<T1,T2>::display(ostream& os) const
{
 if (_basePointer)
   os << '(' << _baseObject << ',' << *_basePointer << ')';
 else
   os << '(' << _baseObject << ',' <<     "NULL"    << ')';
}

//--
template <class T1, class T2>
bool Base<T1,T2>::isEqualTo(const Base<T1,T2>& aB) const
{
 if (_baseObject != aB._baseObject) return false;

 if (_basePointer && aB._basePointer)
 {
  if (*_basePointer != *aB._basePointer) return false;
 }
 else
 {
  if (_basePointer!=aB._basePointer)     return false;
 }

 return true;
}

//--
template <class T1, class T2>
void Base<T1,T2>::_copy(const Base<T1,T2>& aB)
{
 _baseObject = aB._baseObject;
 if (aB._basePointer) _basePointer = new T2(*aB._basePointer);
 else                 _basePointer = NULL;
}

//--
template <class T1, class T2>
void Base<T1,T2>::_destroy(void)
{
 if (_basePointer) delete _basePointer;
}

